package at.fhooe.mc.magicwand.client.gui;

import at.fhooe.mc.magicwand.client.controller.ControlClass;
import at.fhooe.mc.magicwand.client.exceptions.CompassException;
import at.fhooe.mc.magicwand.client.exceptions.ServerConnectionException;
import at.fhooe.mc.magicwand.client.poi.POI;
import java.io.IOException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import de.enough.polish.ui.IconCustomItem;

/**
 * @class public class GuiMIDlet extends MIDlet implements CommandListener
 * @author Maria Neumayer
 * @brief this class contains the gui implementation and uses J2ME polish to
 *        display the forms
 */
public class GuiMIDlet extends MIDlet implements CommandListener {

    /**
     * ControlClass to communicate with other parts of the application
     */
    private ControlClass m_control;
    /**
     * Current Display of the MIDlet
     */
    private Display m_display;
    /**
     * If the MIDlet is paused
     */
    private boolean m_midletPaused = false;
    /**
     * ConnectionScreen, shown at startup
     */
    private Form m_ConnectionScreen;
    /**
     * SplashScreen, shown after ConnectionScreen
     */
    private Form m_splashScreen;
    /**
     * MainScreen, to start request or set options
     */
    private Form m_mainScreen;
    /**
     * OptionsScreen, to set specific options
     */
    private Form m_optionsScreen;
    /**
     * FilterScreen, to set filters to the result
     */
    private Form m_filterScreen;
    /**
     * ViewScreen, to change the view of the result
     */
    private Form m_viewScreen;
    /**
     * LoadScreen, while sending and receiving data
     */
    private Form m_loadScreen;
    /**
     * ResultScreen, shows the result of a request
     */
    private Form m_resultScreen;
    /**
     * Command to exit the application
     */
    private Command m_cmdExit;
    /**
     * Command to confirm
     */
    private Command m_cmdOk;
    /**
     * Command to go back, without change
     */
    private Command m_cmdBack;
    /**
     * Command to get to the OptionsScreen
     */
    private Command m_cmdOptions;
    /**
     * Command to get to the FilterScreen
     */
    private Command m_cmdFilter;
    /**
     * Command to get to the ViewScreen
     */
    private Command m_cmdView;
    /**
     * Command to abort request
     */
    private Command m_cmdAbort;
    /**
     * ChoiceGroup to set Distance
     */
    private ChoiceGroup m_itmDistance;
    /**
     * ChoiceGroup to set Filter
     */
    private ChoiceGroup m_itmFilter;
    /**
     * ChoiceGroup to set View
     */
    private ChoiceGroup m_itmView;
    /**
     * TextField to set Server-Address
     */
    private TextField m_txfAddress;
    /**
     * TextField to set Port
     */
    private TextField m_txfPort;
    /**
     * TextField to set Distance
     */
    private TextField m_txfDistance;
    /**
     * TextField to set max. Results
     */
    private TextField m_txfMaxResults;
    /**
     * String array to set options for search
     */
    private String[] m_options;
    /**
     * String array with selected categories
     */
    private String[] m_categories;
    /**
     * String with selected view
     */
    private String m_view;
    /**
     * Amount of selected categories
     */
    private int m_cCount;

    /**
     * @fn public GuiMIDlet()
     * @brief creates display and options array
     */
    public GuiMIDlet() {
        m_display = Display.getDisplay(this);
    }

    /**
     * @fn public void destroyApp(boolean _unconditional)
     * @param _unconditional
     * @brief destroys the application
     */
    public void destroyApp(boolean _unconditional) {
    }

    /**
     * @fn public void startApp()
     * @brief starts the application
     */
    public void startApp() {
        if (m_midletPaused) {
            resumeMIDlet();
        } else {
            startMIDlet();
        }
        m_midletPaused = false;

    }

    /**
     * @fn public void pauseApp()
     * @brief pauses the application
     */
    public void pauseApp() {
        m_midletPaused = true;
    }

    /**
     * @fn private Form buildConnectionScreen()
     * @return ConnectionScreen
     * @brief creates the ConnectionScreen
     */
    private Form buildConnectionScreen() {
        //#style Form
        Form f = new Form("Connection");

        //#style FormItem
        f.append("Server-address:");
        m_txfAddress = new TextField("", "", 30, TextField.URL);
        //#style TextField
        f.append(m_txfAddress);

        //#style FormItem
        f.append("Port:");
        m_txfPort = new TextField("", "", 30, TextField.NUMERIC);
        //#style TextField
        f.append(m_txfPort);

        m_cmdOk = new Command("Apply", Command.OK, 0);
        m_cmdExit = new Command("Exit", Command.EXIT, 0);

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdExit);

        f.setCommandListener(this);
        m_ConnectionScreen = f;
        return f;
    }

    /**
     * @fn private Form buildSplashScreen()
     * @return splashScreen
     * @brief creates the splashScreen
     */
    private Form buildSplashScreen() {
        //#style SplashForm
        Form f = new Form("Login");

        //#style FormItem
        f.append("Paul Klingelhuber");
        //#style FormItem
        f.append("    Manuel Lachberger");
        //#style FormItem
        f.append("        Maria Neumayer");
        //#style FormItem
        f.append("             Manuela Vogler");
        m_splashScreen = f;
        return f;
    }

    /**
     * @fn private Form buildMainScreen()
     * @return mainScreen
     * @brief creates the mainScreen, where the user can start a search or call
     *        the optionsScren
     */
    private Form buildMainScreen() {
        //#style Form
        Form f = new Form(" ");

        //#style FormItem
        f.append("Point with the mobile phone into the desired direction and select Menu - Start");

        m_cmdExit = new Command("Exit", Command.EXIT, 0);
        m_cmdOk = new Command("Start", Command.OK, 0);
        m_cmdOptions = new Command("Options", Command.ITEM, 0);

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdOptions);
        f.addCommand(m_cmdExit);

        f.setCommandListener(this);
        m_mainScreen = f;
        return f;
    }

    /**
     * @fn private Form buildOptionsScreen()
     * @return optionsScreen
     * @brief creates the OptionsScreen where the user can change the settings
     *        for the search
     */
    private Form buildOptionsScreen() {
        //#style OptionsForm
        Form f = new Form("Search area");

        //#style FormItem
        f.append("max. distance (m):");
        m_txfDistance = new TextField("", "", 30, TextField.NUMERIC);
        //#style TextField
        f.append(m_txfDistance);

        m_itmDistance = new ChoiceGroup("", ChoiceGroup.EXCLUSIVE);
        //#style choiceItem
        m_itmDistance.append("precise", null);
        //#style choiceItem
        m_itmDistance.append("generous", null);
        m_itmDistance.setSelectedIndex(0, true);
        f.append(m_itmDistance);

        //#style FormItem
        f.append("max. number of results:");
        m_txfMaxResults = new TextField("", "", 30, TextField.NUMERIC);
        //#style TextField
        f.append(m_txfMaxResults);

        m_cmdBack = new Command("Back", Command.BACK, 0);
        m_cmdOk = new Command("Apply", Command.OK, 0);

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdBack);

        f.setCommandListener(this);
        m_optionsScreen = f;
        return f;
    }

    /**
     * @fn private Form buildLoadScreen()
     * @return loadScreen
     * @brief creates the loadScreen where the data is sent and received
     */
    private Form buildLoadScreen() {
        //#style Form
        Form f = new Form(" ");

        //#style FormItem
        f.append("Data is established...");

        m_cmdAbort = new Command("Cancel", Command.CANCEL, 0);

        f.addCommand(m_cmdAbort);

        f.setCommandListener(this);
        m_loadScreen = f;
        return f;
    }

    /**
     * @fn private Form buildResultScreen()
     * @return resultScreen
     * @brief creates the resultScreen where the result-data is shown, the user
     *        can make another search, call the filterScreen or the viewScreen
     */
    private Form buildResultScreen() {
        //#style Form
        Form f = new Form("Results");

        m_cmdExit = new Command("Exit", Command.EXIT, 0);
        m_cmdOk = new Command("New search", Command.OK, 0);
        m_cmdFilter = new Command("Filter", Command.ITEM, 0);
        m_cmdView = new Command("Views", Command.ITEM, 0);

        if (m_control.getM_poiOptions().size() != 0) {
            for (int i = 0; i < m_control.getM_poiOptions().size(); i++) {
                String send = ((POI) m_control.getM_poiOptions().elementAt(i)).getM_name();
                if (((POI) m_control.getM_poiOptions().elementAt(i)).getM_category() != null) {
                    send = ((POI) m_control.getM_poiOptions().elementAt(i)).getM_priority() + ". " + send + " - " + ((POI) m_control.getM_poiOptions().elementAt(i)).getM_category();
                }

                try {
                    Image image;
                    if (((POI) m_control.getM_poiOptions().elementAt(i)).getM_group() == 1) {
                        image = Image.createImage("/near.gif");
                    } else if (((POI) m_control.getM_poiOptions().elementAt(i)).getM_group() == 2) {
                        image = Image.createImage("/middle.gif");
                    } else {
                        image = Image.createImage("/far.gif");
                    }

                //#style IconItem
                IconCustomItem text = new IconCustomItem(send, image);
                f.append(text);

                } catch (IOException _ex) {
                    //#style FormItem
                    f.append(send);
                }

                String description = ((POI) m_control.getM_poiOptions().elementAt(i)).getM_description();
                if (description != null) {
                    description = "Description: " + description;
                    //#style FormItem
                    f.append(description);
                }

                String link = ((POI) m_control.getM_poiOptions().elementAt(i)).getM_link();
                if (link != null) {
                    link = " - " + link;
                    //#style FormItem
                    f.append(link);
                }
                //#style FormItem
                f.append("");
            }
        } else {
            //#style FormItem
            f.append("No results in this area");
        }

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdFilter);
        f.addCommand(m_cmdView);
        f.addCommand(m_cmdExit);

        f.setCommandListener(this);
        m_resultScreen = f;
        return f;
    }

    /**
     * @fn private Form buildFilterScreen()
     * @return filterScreen
     * @brief creates the filterScreen where the user can filter the results by
     *        selecting different categories
     */
    private Form buildFilterScreen() {
        //#style Form
        Form f = new Form("Filter");

        m_itmFilter = new ChoiceGroup("", ChoiceGroup.MULTIPLE);

        for (int i = 0; i < m_control.getM_categories().length; i++) {
            //#style choiceItem
            m_itmFilter.append(m_control.getM_categories()[i], null);
        }

        //#style choiceItem
        f.append(m_itmFilter);

        m_cmdBack = new Command("Back", Command.BACK, 0);
        m_cmdOk = new Command("Apply", Command.OK, 0);

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdBack);

        f.setCommandListener(this);
        m_filterScreen = f;
        return f;
    }

    /**
     * @fn private Form buildViewScreen()
     * @return viewScreen
     * @brief creates the viewScreen where the user can change the view of the
     *        result
     */
    private Form buildViewScreen() {
        //#style Form
        Form f = new Form("Views");

        m_itmView = new ChoiceGroup("", ChoiceGroup.EXCLUSIVE);

        //#style choiceItem
        m_itmView.append("Name", null);

        //#style choiceItem
        m_itmView.append("Name, description", null);

        //#style choiceItem
        m_itmView.append("Name, description, link", null);

        m_itmView.setSelectedIndex(0, true);

        f.append(m_itmView);

        m_cmdBack = new Command("Back", Command.BACK, 0);
        m_cmdOk = new Command("Apply", Command.OK, 0);

        f.addCommand(m_cmdOk);
        f.addCommand(m_cmdBack);

        f.setCommandListener(this);
        m_viewScreen = f;
        return f;
    }

    /**
     * @fn public void commandAction(Command _cmd, Displayable _displayable)
     * @param :cmd
     *            triggered command
     * @param :displayable
     *            current display
     * @brief handels the commandActions of the application
     */
    public void commandAction(Command _cmd, Displayable _displayable) {
        if (_cmd == m_cmdExit) { //exit application
            exitMIDlet();
        } else if (_cmd == m_cmdOk) {
            if (_displayable == m_ConnectionScreen) { //set connection info
                if (m_txfAddress.getString().equals("") || m_txfPort.getString().equals("")) {
                    Alert a = new Alert("Error");
                    a.setString("Input not complete");
                    m_display.setCurrent(a, _displayable);
                } else {
                    m_display.setCurrent(buildSplashScreen());
                    m_control.setM_gui(this);
                    openConnection(m_txfAddress.getString(), Integer.parseInt(m_txfPort.getString()));
                }
            } else if (_displayable == m_mainScreen) { //start search
                m_display.setCurrent(buildLoadScreen());
                m_control.setM_gui(this);
                m_control.setM_makeRequest(true);
            } else if (_displayable == m_optionsScreen) { //set options
                m_options = new String[3];

                m_options[0] = m_txfDistance.getString();

                int selected = m_itmDistance.getSelectedIndex();
                m_options[1] = m_itmDistance.getString(selected);

                m_options[2] = m_txfMaxResults.getString();

                if (m_options[0].equals("") || m_options[2].equals("")) {
                    Alert a = new Alert("Error");
                    a.setString("Input not complete");
                    m_display.setCurrent(a, _displayable);
                } else {
                    m_control.setOptions(m_options);
                    m_display.setCurrent(buildMainScreen());
                }
            } else if (_displayable == m_filterScreen) { //set filter
                boolean[] selected = new boolean[m_control.getM_categories().length];
                m_itmFilter.getSelectedFlags(selected);
                m_categories = new String[selected.length];
                m_cCount = 0;
                for (int i = 0; i < selected.length; i++) {
                    if (selected[i] == true) {
                        m_categories[m_cCount++] = m_itmFilter.getString(i);
                    }
                }
                if (m_cCount == 0) {
                    Alert a = new Alert("Error");
                    a.setString("Input not complete");
                    m_display.setCurrent(a, _displayable);
                } else {
                    m_control.makeVector(m_categories, m_view);
                    m_display.setCurrent(buildResultScreen());
                }
            } else if (_displayable == m_viewScreen) { //set view
                int selected = m_itmView.getSelectedIndex();
                m_view = m_itmView.getString(selected);
                if (m_view == null) {
                    Alert a = new Alert("Error");
                    a.setString("Input not complete");
                    m_display.setCurrent(a, _displayable);
                } else {
                    m_control.makeVector(m_categories, m_view);
                    m_display.setCurrent(buildResultScreen());
                }
            } else if (_displayable == m_resultScreen) { //new search
                m_display.setCurrent(buildMainScreen());
            }
        } else if (_cmd == m_cmdBack) {
            if (_displayable == m_optionsScreen) { //don't set options
                m_display.setCurrent(buildMainScreen());
            } else if (_displayable == m_filterScreen) { //don't set categories
                m_display.setCurrent(buildResultScreen());
            } else if (_displayable == m_viewScreen) { //don't set view
                m_display.setCurrent(buildResultScreen());
            }
        } else if (_cmd == m_cmdAbort) { //abort search
            m_control.setM_makeRequest(false);
            m_display.setCurrent(buildMainScreen());
        } else if (_cmd == m_cmdOptions) { //start optionsScreen
            m_display.setCurrent(buildOptionsScreen());
        } else if (_cmd == m_cmdFilter) { //start filterScreen
            m_display.setCurrent(buildFilterScreen());
        } else if (_cmd == m_cmdView) { //start viewScreen
            m_display.setCurrent(buildViewScreen());
        }
    }

    /**
     * @fn public void openConnection(String _address, int _port)
     * @param _address server address
     * @param _port port of socket
     * @brief opens the connections and starts the thread
     */
    public void openConnection(String _address, int _port) {
        m_options = new String[3];
        m_control.openConnection(_address, _port);
    }

    /**
     * @fn public void setMainScreen(Exception _e)
     * @param _e Exceptions while opening connection
     * @brief sets mainScreen after opening connection
     */
    public void setMainScreen(Exception _e) {
        if (_e != null) {
            try {
            if (m_control.isM_connectionOpened()) {
                m_control.exitConnection();
            }
            if (m_control.isM_compassConnOpened()) {
                m_control.exitCompass();
            }
            } catch (Exception _ce) {
                exitMIDlet();
            }
            Alert a = new Alert("Error");
            a.setString(_e.getMessage());
            m_display.setCurrent(a, buildConnectionScreen());
        } else {
            m_display.setCurrent(buildMainScreen());
        }
    }

    /**
     * @fn public void setScreen(Exception _e)
     * @param _e Exceptions while making request
     * @brief sets result- or mainScreen after request
     */
    public void setScreen(Exception _e) {
        if (_e != null) {
            Alert a = new Alert("Error");
            a.setString(_e.getMessage());
            m_display.setCurrent(a, buildMainScreen());
        } else if (m_control.isRequestSuccessful()) { //successful
            m_control.makeVector(null, null);
            m_categories = null;
            m_view = null;
            m_cCount = 0;
            m_display.setCurrent(buildResultScreen());
        } else { //not successful
            m_display.setCurrent(buildMainScreen());
        }
    }

    /**
     * @fn public void resumeMIDlet()
     * @brief Performs an action assigned to the Mobile Device - MIDlet Resumed
     *        point.
     */
    public void resumeMIDlet() {
    }

    /**
     * @fn public void exitMIDlet()
     * @brief Exits MIDlet.
     */
    public void exitMIDlet() {
        try {
            if (m_control.isM_connectionOpened()) {
                m_control.exitConnection();
            }
            if (m_control.isM_compassConnOpened()) {
                m_control.exitCompass();
            }
            m_control.setM_threadFlag(false);
            m_display.setCurrent(null);
            destroyApp(true);
            notifyDestroyed();
        } catch (ServerConnectionException _e) {
            Alert a = new Alert("Error");
            a.setString(_e.getMessage());
            m_display.setCurrent(a, null);
            destroyApp(true);
            notifyDestroyed();
        } catch (CompassException _e) {
            Alert a = new Alert("Error");
            a.setString(_e.getMessage());
            m_display.setCurrent(a, null);
            destroyApp(true);
            notifyDestroyed();
        }
    }

    /**
     * @fn public void startMIDlet()
     * @brief Performs an action assigned to the Mobile Device - MIDlet Started
     *        point.
     */
    public void startMIDlet() {
        m_display.setCurrent(buildConnectionScreen());
        m_control = new ControlClass();
        m_control.start();
    }
}
