/*
 ============================================================================
 Name		: CompassServer.h
 Author	  : Manuela Vogler
 Version	 : 1.0
 Copyright   : 
 Description : CCompassServer declaration
 ============================================================================
 */

#ifndef COMPASSSERVER_H
#define COMPASSSERVER_H

#include <e32base.h>	
#include <e32std.h>		
#include <es_sock.h>
#include <in_sock.h>
#include "CompassReader.h"
#include "MagicWandServerContainer.h"

/**
 * Indicating the server's status.
 */
enum TCompassServerState
	{
	EConnecting, /** Waiting for client */
	EConnected, /** Connection to client established */
	EReceiving, /** Receiving request */
	ESending, /** Sending response */
	EReset /** Connection reset */
	};

/**
 * Maximum length of response message.
 */
const int KAnswerSize = 5;

/**
 * @class CCompassServer CompassServer.h
 * @brief Communicates with a client. Receives a request if a client wants to know the current compass angle and sends a response with this data. Also manages the output of status messages in the GUI.
 */
class CCompassServer : public CActive
	{
public:
	// Constructors and destructor
	~CCompassServer();
	static CCompassServer* NewL();
	static CCompassServer* NewLC();
public:
	//Public functions
	void StartL();	
	void RequestData();
	void Stop();
	void SetContainerReference(CMagicWandServerContainer* aContainer);
	
private:
	//Private construction functions
	CCompassServer();
	void ConstructL();
private:
	//Private functions
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);
	void ProcessData();
private:
	//Instance variables
	
	/**
	 * The server's current status.
	 */
	TCompassServerState iServerState;

	/** 
	 * A connection to the socket server.
	 */
	RSocketServ iSocketServer;

	/** 
	 * A socket to connect with 
	 */
	RSocket iSocket;

	/** 
	 * The buffer to read data to.
	 */
	TBuf8<255> iBuffer;

	/**
	 * The address to be used in the connection.
	 */
	TInetAddr iAddress;

	/**
	 * The port number to connect to.
	 */
	TInt iPort;

	/** 
	 * The socket that connections are accepted to 
	 */
	RSocket iAcceptedSocket;

	/** 
	 * The length of data read.
	 */
	TSockXfrLength iLen;

	/**
	 * The active socket.
	 */
	RSocket* iActiveSocket;

	/**
	 * The compass reader used to get the compass angle.
	 */
	CCompassReader* iReader;

	/**
	 * The response message which is sent to the client.
	 */
	HBufC8* iMessage;

	/**
	 * Reference to the GUI container, used for the output of status messages.
	 */
	CMagicWandServerContainer* iContainer;
	};

#endif // COMPASSSERVER_H
